// Datei: rs232lcd.c
//
// Ansteuerung von LCD-Anzeigen ber RS232
// ohne externe Stromversorgung
//
// Holger Klabunde 
// 30.06.2001
// Compiler CC5x

#include "prozess.h"      // Der verwendete Prozessor

#include "protos.h"
#include "lcd.h"

//CC5x kann nicht mehrere Objektdateien erzeugen und dann zusammenlinken
//Deshalb diese INCLUDES der *.c Module
#include "serial.c"
#include "delay.c"
#include "lcd.c"

void SetPosition();
void SendID();
void SetCursor();
void ShiftDisplay();
void SetCGRam();
void CGChar();
 
//*******************************************************
// Hauptprogramm
//*******************************************************
void main()
{
 unsigned char by;
//1=Eingang
//0=Ausgang
 
 CMCON=0b00000111;

 CLRWDT;
 OPTION=0b10000010; //Pullups off,Prescaler auf TMR0, 1:8
                    
 TRISA=0b11100000;   
 TRISB=0b00000010;   //RB2>TX, RB1<RX
 PORTA=0x00;

 InitUSART(); //Serielle Schnittstelle einstellen
 GIE=1;       // Interrupts erlauben
 PEIE=1;      // Periphere Interrupts an

 lcdport=0;
 LCDInit();
 LCDWriteChar('T');
 LCDWriteChar('e');
 LCDWriteChar('s');
 LCDWriteChar('t');
 LCDWriteChar('1');
 LCDPos(2,1);
 LCDWriteChar('T');
 LCDWriteChar('e');
 LCDWriteChar('s');
 LCDWriteChar('t');
 LCDWriteChar('2');
 LCDPos(3,1);
 LCDWriteChar('T');
 LCDWriteChar('e');
 LCDWriteChar('s');
 LCDWriteChar('t');
 LCDWriteChar('3');
 LCDPos(4,1);
 LCDWriteChar('T');
 LCDWriteChar('e');
 LCDWriteChar('s');
 LCDWriteChar('t');
 LCDWriteChar('4');

 RB3=1;         //Hintergrundbeleuchtung an
 
 while(1) // Programm-Endlosschleife
  {
   if(rcnt>0) //Irgendwas empfangen ?
    {
     by=Receive(); //Byte abholen  

     if(by=='~') //Hier kommt ein Befehl
      {
       by=Receive(); //Befehl abholen
         
       switch(by) //Befehlscode auswerten
        {
         case 'x': LCDCommand(CLEAR_DISPLAY); Wait(); //Clear Screen
                   break;        
         case 'i': LCDInit();              //LCD Display Software Reset
                   RB3=1;
                   break;        
         case 'p': SetPosition(); break;   //Cursor auf neue Position
         case 'v': SendID(); break;        //Versionsnummer zurcksenden,
                                           //gut um zu sehen ob das Display an diesem COM Port hngt
         case 'c': SetCursor(); break;     //Cursormode setzen
         case 'o': LCDCommand(OFF_DISPLAY); break; //Anzeige abschalten, nicht den Controller !
         case 's': ShiftDisplay(); break;  //Die ganze Anzeige um eine Stelle links oder rechts verschieben
         case 'r': SetCGRam(); break;      //Schreibt eigene Zeichen ins CGRam
         case 'z': CGChar(); break;        //Zeigt ein Zeichen aus dem CGRam an
        }
      } //if by=='~'
     else LCDWriteChar(by);                //War Text, dann gleich schreiben

    } //if rcnt>0

  } //while(1) Endlosschleife

}

void CGChar()
{
 unsigned char by;
 
 by=Receive(); //Adresse abholen
 by-='0';      //Bin aus ASCII machen
 LCDWriteByte(by); //Zeichen anzeigen
}

void SetCGRam()
{
 unsigned char by,i,buf;
 
 by=Receive(); //Adresse abholen
 by-='0';      //Bin aus ASCII machen

 LCDCGAdr(by); //CG_RAM Adresse setzen

 for(i=0; i<8; i++)
  {
   by=Receive(); //einfach nur das Komma empfangen, aber nicht auswerten
   by=Receive(); //Zehnerstelle des Zeichens abholen
   by-='0';      //Bin aus ASCII machen
   buf=10*by;
   by=Receive(); //Einerstelle des Zeichens abholen
   by-='0';      //Bin aus ASCII machen
   buf+=by;
   LCDWriteByte(buf); //und schreiben
  }
  
 LCDCommand(0x80); //DD_RAM Adresse 0 setzen, besser ist das
}

void ShiftDisplay()
{
 unsigned char by;
 
 by=Receive(); //Richtung abholen

 switch(by)
  {
   case 'l': LCDCommand(SHIFT_LEFT); break;   //Display links schieben
   case 'r': LCDCommand(SHIFT_RIGHT); break;  //Display rechts schieben
  }
}

void SetCursor()
{
 unsigned char by;
 
 by=Receive(); //Cursormodus abholen

 switch(by)
  {
   case '0': LCDCommand(CURSOR_OFF); break;   //Cursor abschalten
   case 'o': LCDCommand(CURSOR_ON); break;    //Cursor einschalten. Cursor ist statisch
   case 'b': LCDCommand(CURSOR_BLINK); break; //Cursor und Zeichen an der Stelle blinken
  }
}

void SetPosition()
{
 unsigned char x,y,by;
 
 by=Receive(); //Zeilennummer abholen
 x=by-'0';     //BIN aus ASCII machen

 by=Receive(); //einfach nur das Komma empfangen, aber nicht auswerten
 
 by=Receive(); //Zehnerstelle der Spalte empfangen
 y=by-'0';     //BIN aus ASCII machen
 y=y*10;       //und mal 10 nehmen

 by=Receive(); //Einerstelle der Spalte empfangen
 by=by-'0';    //BIN aus ASCII machen
 y+=by;        //und addieren

 LCDPos(x,y);  //Neue Cursorposition einstellen
}

void SendID()
{
 Transmit('L');
 Transmit('C');
 Transmit('D');
 Transmit('1');
 Transmit(0x0d);
 Transmit(0x0a);
}