// Datei: Serial.c
//
// Serielle Datenbertragung
//
// In Empfangsrichtung wird ein Interrupt mit Ringpuffer benutzt !
// Die Routinen stammen von SDCC und sind eigentlich fr einen
// 8051 Controller gedacht. Geht ohne groe nderungen auch
// mit einem PIC.
//
// Holger Klabunde 
// 26.06.2001
// Compiler CC5x

#include "prozess.h"      // Der verwendete Prozessor
#include "protos.h"

#include <int16CXX.H>


#define RBUFLEN 16 //Gre des Empfangspuffers
unsigned char rbuf[RBUFLEN]; //Ringpuffer fr Empfang

unsigned char buf;
unsigned char rpos,rcnt;

#pragma origin 4  //Interruptvektor vom PIC

interrupt int_server( void)
{
//Alle wichtigen Register retten
 int_save_registers    // W, STATUS (and PCLATH)
 char sv_FSR = FSR;  // save FSR if required

//###############################
//Beginn RS232-Empfangs-Interrupt
//###############################
 if(RCIF)
  {
   if(rcnt<RBUFLEN) //noch Platz im Puffer ?
    {
     buf=rpos+rcnt;     //Ring Position im Array berechnen
     buf=buf % RBUFLEN; //Position auf jeden Fall im Array
     rcnt++;            //empfangene Bytes zhlen
     rbuf[buf] = RCREG; //Daten abholen
    }
   else buf=RCREG; //Sicherheitshalber RCREG lesen wenn Puffer voll ist
                   //sonst wird OERR gesetzt und der Empfang wird blockiert 

   //RCIF wird beim lesen von RCREG zurckgesetzt !
  }
//###############################
//Ende RS232-Empfangs-Interrupt
//###############################

//############################
//Beginn EEPROM-Interrupt
//############################
/*
 if(EEIF) //War ein EEPROM Interrupt
  {
   EEIF=0; //Interrupt Flag lschen
   EEIE=0; //Interrupt selbst lschen
 	  //Quasi als Schreibschutz frs EEPROM
  }
*/
//############################
//Ende EEPROM-Interrupt
//############################

 FSR = sv_FSR;       // restore FSR if saved

//Alle wichtigen Register wiederherstellen
 int_restore_registers // W, STATUS (and PCLATH)
}

//#######################################
//Serielle Schnittstelle einstellen 
//9600Baud ,8 Bit ,kein Parity, 1 Stopbit
//#######################################
void InitUSART()
{
 BRGH=1;

 rcnt=0;
 rpos=0;

// SPBRG = (FOSC / (16 * BAUDRATE)) - 1 
// SPBRG=127; // (1200 baud @ 2,4576MHz input clock)
// SPBRG=63; // (2400 baud @ 2,4576MHz input clock)
// SPBRG=31; // (4800 baud @ 2,4576MHz input clock)
 SPBRG=15; // (9600 baud @ 2,4576MHz input clock)
// SPBRG=7; // (19200 baud @ 2,4576MHz input clock)
// SPBRG=207; // (1200 baud @ 4MHz input clock)
// SPBRG=103; // (2400 baud @ 4MHz input clock)
// SPBRG=51; // (4800 baud @ 4MHz input clock)
// SPBRG=25; // (9600 baud @ 4MHz input clock)
// SPBRG=12; // (19200 baud @ 4MHz input clock)

 SPEN = 1; //  Set_Serial_Pins;

// Das mag CC5x nicht obwohl es definiert ist !
// SYNC = 0; //  Set_Async_Mode;
 TXSTA.4 = 0; //Dann eben so ;)

 TX9 = 0; //  Set_8bit_Tx;
 RX9 = 0; //  Set_8bit_Rx;
 CREN = 1; //  Enable_Rx;
 TXEN = 1; //  Enable_Tx;

 RCIE=1;   // Rx Interrupt an
}

//#############################
//Ein Datenbyte empfangen
//bzw. aus dem Ringpuffer holen
//#############################
unsigned char Receive()
{
 unsigned char c;

 while (!rcnt);   /* wait for character */

 RCIE=0;   //Disable Rx Interrupt

 rcnt--;   //ein Byte aus dem Puffer geholt
 c = rbuf [rpos++];
 if (rpos >= RBUFLEN) rpos = 0; //Ringzhler neu setzen

 RCIE=1;   //Enable Rx Interrupt

 return (c);
}

//######################
//Ein Datenbyte senden
//######################
void Transmit(unsigned char dat)
{
 while(!TXIF);   // Loop till TXREG empty, WDT off !

 TXREG = dat; // Send data
}
