// Datei: LCD.C
//
// Ansteuerung der LCD-Anzeige
// im 4 Bit Daten Mode
//
// Holger Klabunde
// 30.06.2001
// Compiler CC5x

#include "prozess.h"      /* Der verwendete Prozessor */
#include "lcd.h"
#include "protos.h"

unsigned char lcdport;

//*******************************************
// Schreibt ein Datenbyte in die LCD-Anzeige
// Schreibt das Byte in zwei 4 Bit Nibbles
//*******************************************
void LCDWriteByte(unsigned char data)
{
 unsigned char buff;
 

 lcdport=PORTB;   //alten Wert merken
 
 LCDCS_OFF();

 buff=data>>4;   //High Nibble des Bytes
 lcdport=lcdport&0x0F;	//Hier D7-D4 ersma lschen zum
 lcdport|=(buff<<4);	//reinodern vom High Nibble

 LCDCS_ON();	      //High Nibble schreiben
 LCDCS_OFF();	

 buff=data&0x0F;        //Low Nibble des Bytes
 lcdport=lcdport&0x0F;	//D7-D4 ersma lschen zum
 lcdport|=(buff<<4);		//reinodern vom Low Nibble

 LCDCS_ON();	      //Low Nibble schreiben
 LCDCS_OFF();	
 Delay100us(1);       //etwas auf die Anzeige warten
}

//**************************************************
//Schreibt einen Buchstaben in die LCD-Anzeige
//**************************************************
void LCDWriteChar(char ps)
{
 if(ps<' ') return; //Codes kleiner Space besser nicht benutzen
 
 //bersetzungstabelle fr deutsche Umlaute
 switch(ps)
  {
   case '' : ps=0xE1; break; //Umlenkung auf neuen Zeichen Code
   case '' : ps=0xEF; break;
   case '' : ps=0xF5; break;
   case '' : ps=0xE2; break;
   case '' : ps=0x00; break; //Umlenkung ins Character RAM
   case '' : ps=0x01; break;
   case '' : ps=0x02; break;
   case '' : ps=0xDF; break;
   case '' : ps=0xE4; break;
  }
     
 LCDWriteByte(ps);
}

//****************************************************
// Springt auf Zeile x und Position y der LCD-Anzeige 
// Gltig bis 4 Zeilen bei 4x20.
// 4x24 oder 4x40 ??
//****************************************************
void LCDPos(unsigned char zeile,unsigned char position)
{
 unsigned char buff;

// 0x80 wegen Display Adress Set.
// Erste Position ist bei mir 1, nicht 0.
// Offset's
// Zeile 1 -> 0x80-0x01=0x7F
// Zeile 2 -> 0x80+0x20-0x01=0x9F 
// Zeile 3 -> 0x80+0x40-0x01=0xBF 
// Zeile 4 -> 0x80+0x60-0x01=0xDF 
 
 switch(zeile)
  {
   case 1: buff=0x7F+position; break;
   case 2: buff=0x9F+position; break;
   case 3: buff=0xBF+position; break;
   case 4: buff=0xDF+position; break;
   default : buff=0x80; break; //Ungltige Position. Neue => 1,1
  }

 LCDCommand(buff); 
}

//*****************************************
// Commando ans Display senden
//*****************************************
void LCDCommand(unsigned char mode)
{
 LCDRS_OFF(); //Controlregister
 LCDWriteByte(mode);
 LCDRS_ON(); //Zurck auf Datenregister
}

//*****************************************
// CGRam Adresse setzen
//*****************************************
void LCDCGAdr(unsigned char adr)
{
 unsigned char buf;
 buf=adr<<3;        //Adresse * 8
 buf+=0x40;         //CGRam Flag setzen
 LCDCommand(buf);
}

//*********************************************
//Stellt die Funktionsweise der LCD Anzeige ein
//*********************************************
void LCDInit()
{
 int i;

 LCDRS_OFF();  //Control Register
 LCDCS_OFF();  //LCD-Anzeige abwhlen

 Delay1ms(50); //Mglichen Power On Reset abwarten
 for(i=0; i<3; i++)
  {
   lcdport=lcdport&0x0F;//D7-D4 ersma lschen
   lcdport|=0x30;	//Wert reinodern
   PORTB=lcdport;	//Auf den Port legen

   LCDCS_ON();	//High Nibble schreiben
   LCDCS_OFF(); //LCD-Anzeige abwhlen
   Delay1ms(50); //Ein bichen warten auf die Anzeige
  }

//Der erste Schreibzugriff nach dem Reset ist ein 8 Bit
//Zugriff!! Also nur einmal schreiben, nicht Low-High Nibble.

//Um die 4Bit Operation einzuschalten
 lcdport=0x20;	//4 Bit Operation
 PORTB=lcdport;	//Auf den Port legen
 LCDCS_ON();	//High Nibble schreiben
 LCDCS_OFF();	//High Nibble schreiben
 Wait();

//Die Displays kmmern sich scheinbar nicht um das Bit fr
//die Anzahl der Zeilen. Bei mir klappte es so mit ein- und
//zweizeiligen Displays
 LCDWriteByte(0x28);    //4 Bit Operation,4 Line,5x7 Font
 Wait();
 LCDWriteByte(0x0C); 	//Display on, Cursor off ,Cursor no blink
 Wait();
 LCDCommand(CLEAR_DISPLAY);
 Wait();
 LCDWriteByte(0x06); 	//Increment, Display Freeze
 Wait();

/*
 LCDWriteByte(0x14); 	//Cursor Move,Right Shift
 Wait();


 LCDWriteByte(0x02); 	//Cursor Home
 Wait();
 LCDRS_ON();		//LCD-Datenregister
*/
 
//Die groen deutschen Umlaute ,, ins Character RAM CGRAM schreiben
//Standard LCD-Displays haben die nicht im Zeichensatz

//Der interne Adresszhler des LCD-Displays luft
//nach jedem Schreibzugriff automatisch hoch 

 LCDCGAdr(0); //CG_RAM Adresse(0) setzen

 for(i=0; i<CGRAM_LEN; i++) //8 Codes pro Zeichen bertragen
  {
   LCDWriteByte(chardefs[i]);
  }

 LCDCommand(0x80); //DD_RAM Adresse 0 setzen, besser ist das
}
